import { slidingWindow, range } from '@virchual/utils/index';
const LAZY_CLASSNAME = 'virchual__lazy';
const LOADING_CLASSNAME = `${LAZY_CLASSNAME}--loading`;
const ERROR_CLASSNAME = `${LAZY_CLASSNAME}--failed`;
const COMPLETE_CLASSNAME = `${LAZY_CLASSNAME}--loaded`;
/**
 * Return true if given elemen is a <picture> tag.
 *
 * @param element HTML Element.
 */
function isPictureTag(element) {
    return element && element.nodeName.toLowerCase() === 'picture';
}
/**
 * Return either parent <picture> tag or <img> tag.
 *
 * @param element Image HTML tag.
 */
function getImage(image) {
    const picture = image.parentNode;
    if (isPictureTag(picture)) {
        return picture;
    }
    return image;
}
export class LazyLoadImage {
    constructor(imports, settings) {
        this.imports = imports;
        const { lazyload, lazyloadSelector } = Object.assign({ lazyload: true, lazyloadSelector: 'img,picture' }, settings);
        this.lazyload = lazyload;
        this.lazyloadSelector = lazyloadSelector;
        this.events = {
            load: this.onLoad.bind(this),
            error: this.onError.bind(this),
        };
        // exit in case lazy loading is disabled
        if (!this.lazyload) {
            return;
        }
        imports.eventBus.on({
            mounted: () => {
                this.doLazyLoad();
            },
            move: () => {
                this.doLazyLoad();
            },
        });
    }
    /**
     * Lazy load images.
     */
    doLazyLoad() {
        const images = this.getImages();
        images.forEach(image => this.loadImage(image));
    }
    /**
     * Get all image tags (<img>, <picture>) to lazy load.
     */
    getImages() {
        const indices = range(0, this.imports.virchual.slides.length - 1);
        const slidesWindowIndices = slidingWindow(indices, this.imports.virchual.currentIndex, 1);
        const images = [];
        slidesWindowIndices.forEach(index => {
            const slide = this.imports.virchual.slides[index];
            if (!slide.isMounted) {
                return;
            }
            [].forEach.call(slide.ref.querySelectorAll(this.lazyloadSelector), (img) => {
                // skip <img> tags within <picture> tags
                if (isPictureTag(img.parentElement)) {
                    return;
                }
                images.push(img);
            });
        });
        return images;
    }
    /**
     * Do the actual lazy loading for a given image.
     *
     * @param image Either <img> or <picture> tag.
     */
    loadImage(image) {
        let hasLazyImages = false;
        [].forEach.call(image.querySelectorAll('img,source'), (source) => {
            const srcSetData = source.dataset.srcset;
            const srcData = source.dataset.src;
            if (srcSetData) {
                source.setAttribute('srcset', srcSetData);
                source.removeAttribute('data-srcset');
            }
            // source is <img>
            if (srcData) {
                source.setAttribute('src', source.dataset.src);
                source.removeAttribute('data-src');
                this.imports.eventBus.on(this.events, source);
            }
            if (srcSetData || srcData) {
                hasLazyImages = true;
            }
        });
        if (hasLazyImages) {
            image.classList.add(LAZY_CLASSNAME);
            image.classList.add(LOADING_CLASSNAME);
        }
    }
    onLoad(event) {
        const image = event.target;
        this.completeLoading(image);
    }
    onError(event) {
        const image = event.target;
        const target = getImage(image);
        target.classList.add(ERROR_CLASSNAME);
        this.completeLoading(image);
    }
    completeLoading(image) {
        const target = getImage(image);
        target.classList.remove(LOADING_CLASSNAME);
        target.classList.add(COMPLETE_CLASSNAME);
        this.imports.eventBus.off(this.events, image);
    }
}
//# sourceMappingURL=lazyload-image.js.map