(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.Virchual = factory());
}(this, (function () { 'use strict';

  function stop(event) {
      if (!event)
          return;
      event.stopImmediatePropagation();
      event.stopPropagation();
      event.preventDefault();
  }
  class Event {
      constructor() {
          /**
           * Store all event this.data.
           */
          this.handlers = [];
      }
      on(events, handler, element, opts = {}) {
          if (typeof events === 'string') {
              events.split(' ').forEach(event => this.addEvent(event, element, handler, opts));
          }
          else {
              for (const event in events) {
                  this.addEvent(event, handler, events[event], opts);
              }
          }
      }
      off(events, element) {
          if (typeof events === 'string') {
              events.split(' ').forEach(event => this.removeEvent(event, element));
          }
          else {
              for (const event in events) {
                  this.removeEvent(event, element);
              }
          }
      }
      /**
       * Emit an event.
       * This method is only for custom events.
       *
       * @param event - An event name.
       * @param args  - Any number of arguments passed to handlers.
       */
      emit(event, ...args) {
          this.handlers.forEach(item => {
              if (!item.elm && item.event.split('.')[0] === event) {
                  item.handler(...args);
              }
          });
      }
      /**
       * Clear event this.data.
       */
      destroy() {
          this.handlers.forEach(this.unroll);
          this.handlers = [];
      }
      /**
       * Remove the registered event listener.
       *
       * @param item - An object containing event this.data.
       */
      unroll(item) {
          item.elm && item.elm.removeEventListener(item.event, item.handler, item.opts);
      }
      addEvent(event, element, handler, opts) {
          element && element.addEventListener(event, handler, opts);
          this.handlers.push({ event, handler: handler, elm: element, opts: opts });
      }
      removeEvent(event, element) {
          this.handlers = this.handlers.filter(item => {
              if (item && item.event === event && item.elm === element) {
                  this.unroll(item);
                  return false;
              }
              return true;
          });
      }
  }

  class Controls {
      constructor(imports, settings) {
          this.imports = imports;
          this.settings = settings;
          this.controls = [].slice.call(imports.virchual.container.querySelectorAll('.virchual__control'));
          imports.eventBus.on('destroy', () => {
              console.log('controls component destroy');
          });
          this.onClickBound = this.onClick.bind(this);
          this.mount();
      }
      mount() {
          this.controls.forEach(button => this.imports.eventBus.on('click', this.onClickBound, button));
      }
      onClick(event) {
          stop(event);
          const button = event.target.closest('button');
          const control = button.dataset.controls;
          if (control === 'prev') {
              this.imports.virchual.prev();
              return;
          }
          this.imports.virchual.next();
      }
  }

  function createElement(tagName, { classNames, html }) {
      const element = document.createElement(tagName);
      element.className = classNames;
      element.innerHTML = html || '';
      return element;
  }
  /**
   * Remove a given element from a DOM tree.
   *
   * @param elms - Element(s) to be removed.
   */
  function remove(element) {
      if (element && element.parentElement) {
          element.parentElement.removeChild(element);
      }
  }
  /**
   * Append a child to a given element.
   *
   * @param parent - A parent element.
   * @param child  - An element to be appended.
   */
  function append(parent, child) {
      if (parent) {
          parent.appendChild(child);
      }
  }
  /**
   * Prepend an element to parent.
   *
   * @param element- An element to prepend.
   * @param element - A reference element.
   */
  function prepend(parent, element) {
      if (parent && parent.firstChild && element) {
          parent.insertBefore(element, parent.firstChild);
      }
  }
  /**
   * Add or remove css class to given element.
   *
   * @param element Element to add class tp.
   * @param klass CSS classname.
   * @param remove Remove class instead of adding.
   */
  function addOrRemoveClass(element, klass, remove = false) {
      if (!remove) {
          element.classList.add(klass);
          return;
      }
      element.classList.remove(klass);
  }

  /**
   * Check existence of the given object and throw an error if it doesn't.
   *
   * @throws {Error}
   *
   * @param subject - A subject to be confirmed.
   * @param message - An error message.
   */
  function assert(subject, message) {
      if (!subject) {
          throw new Error(message);
      }
  }

  /**
   * Handles debouncing of events via requestAnimationFrame
   *
   * @param callback The callback to handle whichever event
   */
  function debounce(callback, delay = 100) {
      let timeoutId;
      return (...args) => {
          clearTimeout(timeoutId);
          timeoutId = setTimeout(() => requestAnimationFrame(() => {
              timeoutId = null;
              callback(...args);
          }), delay);
      };
  }

  function range(start, end) {
      return Array(end - start + 1)
          .fill(0)
          .map((_, idx) => start + idx);
  }
  /*
   * Rewind the given index if it's out of range.
   *
   * @param index - An index.
   * @param edge - Edge
   *
   * @return A rewound index.
   */
  function rewind(index, edge) {
      if (index > edge) {
          return 0;
      }
      if (index < 0) {
          return edge;
      }
      return index;
  }
  // eslint-disable-next-line @typescript-eslint/no-empty-function
  const noop = () => { };

  /**
   * Returns a sliding window section of an array.
   *
   * @param source Source array.
   * @param start The start of the specified portion of the array. Acts as the center of the sliding window.
   * @param len The window size/length. Specifies how many items the window has on each sides.
   */
  function slidingWindow(source, start, len) {
      const left = range(start - len, start - 1).map(index => get(source, index));
      const center = [source[start]];
      const right = range(start + 1, start + len).map(index => get(source, index));
      const window = [...left, ...center, ...right];
      return window;
  }
  /**
   * Get array item by index.
   *
   * Returning items from the end of the source array when accessing negative indexes or
   * from the start of source array when accessing out of range indexes.
   *
   * @param source Source array.
   * @param index Index of array item to access.
   */
  function get(source, index) {
      if (source.length === 0) {
          return;
      }
      const isOutOfLowerBounds = index < 0;
      const isOutOfUpperBounds = index > source.length - 1;
      const isOutOfBounds = isOutOfLowerBounds || isOutOfUpperBounds;
      const sign = isOutOfUpperBounds ? -1 : +1;
      const multiple = Math.max(Math.floor(Math.abs(index) / source.length), 1);
      if (isOutOfBounds) {
          index += sign * multiple * source.length;
      }
      return source[index];
  }

  const LAZY_CLASSNAME = 'virchual__lazy';
  const LOADING_CLASSNAME = `${LAZY_CLASSNAME}--loading`;
  const ERROR_CLASSNAME = `${LAZY_CLASSNAME}--failed`;
  const COMPLETE_CLASSNAME = `${LAZY_CLASSNAME}--loaded`;
  /**
   * Return true if given elemen is a <picture> tag.
   *
   * @param element HTML Element.
   */
  function isPictureTag(element) {
      return element && element.nodeName.toLowerCase() === 'picture';
  }
  /**
   * Return either parent <picture> tag or <img> tag.
   *
   * @param element Image HTML tag.
   */
  function getImage(image) {
      const picture = image.parentNode;
      if (isPictureTag(picture)) {
          return picture;
      }
      return image;
  }
  class LazyLoadImage {
      constructor(imports, settings) {
          this.imports = imports;
          const { lazyload, lazyloadSelector } = Object.assign({ lazyload: true, lazyloadSelector: 'img,picture' }, settings);
          this.lazyload = lazyload;
          this.lazyloadSelector = lazyloadSelector;
          this.events = {
              load: this.onLoad.bind(this),
              error: this.onError.bind(this),
          };
          // exit in case lazy loading is disabled
          if (!this.lazyload) {
              return;
          }
          imports.eventBus.on({
              mounted: () => {
                  this.doLazyLoad();
              },
              move: () => {
                  this.doLazyLoad();
              },
          });
      }
      /**
       * Lazy load images.
       */
      doLazyLoad() {
          const images = this.getImages();
          images.forEach(image => this.loadImage(image));
      }
      /**
       * Get all image tags (<img>, <picture>) to lazy load.
       */
      getImages() {
          const indices = range(0, this.imports.virchual.slides.length - 1);
          const slidesWindowIndices = slidingWindow(indices, this.imports.virchual.currentIndex, 1);
          const images = [];
          slidesWindowIndices.forEach(index => {
              const slide = this.imports.virchual.slides[index];
              if (!slide.isMounted) {
                  return;
              }
              [].forEach.call(slide.ref.querySelectorAll(this.lazyloadSelector), (img) => {
                  // skip <img> tags within <picture> tags
                  if (isPictureTag(img.parentElement)) {
                      return;
                  }
                  images.push(img);
              });
          });
          return images;
      }
      /**
       * Do the actual lazy loading for a given image.
       *
       * @param image Either <img> or <picture> tag.
       */
      loadImage(image) {
          let hasLazyImages = false;
          [].forEach.call(image.querySelectorAll('img,source'), (source) => {
              const srcSetData = source.dataset.srcset;
              const srcData = source.dataset.src;
              if (srcSetData) {
                  source.setAttribute('srcset', srcSetData);
                  source.removeAttribute('data-srcset');
              }
              // source is <img>
              if (srcData) {
                  source.setAttribute('src', source.dataset.src);
                  source.removeAttribute('data-src');
                  this.imports.eventBus.on(this.events, source);
              }
              if (srcSetData || srcData) {
                  hasLazyImages = true;
              }
          });
          if (hasLazyImages) {
              image.classList.add(LAZY_CLASSNAME);
              image.classList.add(LOADING_CLASSNAME);
          }
      }
      onLoad(event) {
          const image = event.target;
          this.completeLoading(image);
      }
      onError(event) {
          const image = event.target;
          const target = getImage(image);
          target.classList.add(ERROR_CLASSNAME);
          this.completeLoading(image);
      }
      completeLoading(image) {
          const target = getImage(image);
          target.classList.remove(LOADING_CLASSNAME);
          target.classList.add(COMPLETE_CLASSNAME);
          this.imports.eventBus.off(this.events, image);
      }
  }

  class Drag {
      constructor(frame, { event }) {
          this.frame = frame;
          // Determine whether slides are being dragged or not.
          this.isDragging = false;
          this.eventBus = event;
          this.eventBindings = {
              onStart: this.onStart.bind(this),
              onMove: debounce(this.onMove.bind(this), 1),
              onEnd: this.onEnd.bind(this),
          };
      }
      mount() {
          this.eventBus.on('touchstart mousedown', this.eventBindings.onStart, this.frame);
          this.eventBus.on('touchmove mousemove', this.eventBindings.onMove, this.frame, { passive: false });
          this.eventBus.on('touchend touchcancel mouseleave mouseup dragend', this.eventBindings.onEnd, this.frame);
      }
      /**
       * Called when the track starts to be dragged.
       */
      onStart(event) {
          stop(event);
          if (!this.isDragging) {
              this.startInfo = this.analyze(event, {});
              this.currentInfo = this.startInfo;
              this.eventBus.emit('dragstart', this.currentInfo);
          }
      }
      onMove(event) {
          stop(event);
          if (!this.startInfo) {
              return;
          }
          this.currentInfo = this.analyze(event, this.startInfo);
          if (this.isDragging) {
              stop(event);
              this.eventBus.emit('drag', this.currentInfo);
          }
          else {
              if (this.shouldMove(this.currentInfo)) {
                  this.eventBus.emit('drag', this.currentInfo);
                  this.isDragging = true;
              }
          }
      }
      /**
       * Determine whether to start moving the track or not by drag angle.
       *
       * @param info - An information object.
       *
       * @return True if the track should be moved or false if not.
       */
      shouldMove({ offset }) {
          const angle = (Math.atan(Math.abs(offset.y) / Math.abs(offset.x)) * 180) / Math.PI;
          const dragAngleThreshold = 45;
          return angle < dragAngleThreshold;
      }
      /**
       * Called when dragging ends.
       */
      onEnd() {
          this.startInfo = null;
          if (this.isDragging) {
              this.goTo(this.currentInfo);
              this.isDragging = false;
          }
      }
      /**
       * Go to the slide determined by the analyzed data.
       *
       * @param info - An info object.
       */
      goTo(info) {
          const velocity = info.velocity['x'];
          const absV = Math.abs(velocity);
          if (absV > 0) {
              this.eventBus.emit('dragend', this.currentInfo);
          }
      }
      /**
       * Analyze the given event object and return important information for handling swipe behavior.
       *
       * @param event          - Touch or Mouse event object.
       * @param startInfo  - Information analyzed on start for calculating difference from the current one.
       *
       * @return - An object containing analyzed information, such as offset, velocity, etc.
       */
      analyze(event, startInfo) {
          const { timeStamp, touches } = event;
          const { clientX, clientY } = touches ? touches[0] : event;
          const { x: fromX = clientX, y: fromY = clientY } = startInfo.to || {};
          const startTime = startInfo.time || 0;
          const offset = { x: clientX - fromX, y: clientY - fromY };
          const duration = timeStamp - startTime;
          const velocity = { x: offset.x / duration, y: offset.y / duration };
          return {
              offset,
              velocity,
              to: { x: clientX, y: clientY },
              time: timeStamp,
              control: velocity.x < 0 ? 'next' : 'prev',
          };
      }
  }

  /**
   * Map current index to bullet elements index.
   *
   * @param index Current index.
   * @param center Center index of bullets (5 bullets -> center: 2).
   * @param total Total bullets. Same as amount of slides.
   */
  function mapActiveIndex(index, center, total) {
      return index - Math.max(index - center, 0) + Math.max(index - (-1 + total - center), 0);
  }
  /**
   * Return true if bullet is edge bullet.
   *
   * @param index Index of bullet.
   * @param realIndex Real slide index which bullet represents.
   * @param bullets Amount of shown bullets.
   * @param total Total bullets. Same as amount of slides.
   */
  function isEdgeBullet(index, realIndex, bullets, total) {
      const isRightEdge = index === bullets - 1;
      // bullet is either left or right edge
      if (index === 0) {
          return index !== realIndex;
      }
      if (isRightEdge) {
          return realIndex + 1 < total;
      }
      return false;
  }
  /**
   * Get real index of bullet.
   *
   * @param index Index of bullet.
   * @param currentIndex Current active index.
   * @param activeBulletIndex Index of active bullet element.
   */
  function getRealIndex(index, currentIndex, activeBulletIndex) {
      return currentIndex - activeBulletIndex + index;
  }
  class Pagination {
      constructor(container, totalSlides, { diameter, bullets } = {}) {
          this.container = container;
          this.totalSlides = totalSlides;
          this.currentIndex = 0;
          this.ref = container.querySelector('.virchual__pagination');
          this.bulletsLength = Math.min(totalSlides, bullets !== null && bullets !== void 0 ? bullets : 5);
          this.diameter = diameter !== null && diameter !== void 0 ? diameter : 16;
          this.centerIndex = Math.floor(this.bulletsLength / 2);
      }
      render() {
          this.ref = createElement('div', { classNames: 'virchual__pagination' });
          this.ref.style.width = `${this.bulletsLength * this.diameter}px`;
          this.ref.style.height = `${this.diameter}px`;
          range(0, Math.min(this.bulletsLength, this.totalSlides) - 1).forEach(index => {
              const isEdge = isEdgeBullet(index, index, this.bulletsLength, this.totalSlides);
              const bullet = this.renderBullet(index, { isEdge, isActive: index === this.currentIndex });
              append(this.ref, bullet);
          });
          append(this.container, this.ref);
      }
      next() {
          this.goTo(+1);
      }
      prev() {
          this.goTo(-1);
      }
      goTo(sign) {
          this.currentIndex = rewind(this.currentIndex + sign, this.totalSlides - 1);
          const mappedActiveIndex = mapActiveIndex(this.currentIndex, this.centerIndex, this.totalSlides);
          const removeBullet = mappedActiveIndex === this.centerIndex && this.currentIndex > this.centerIndex;
          const removeBulletIndex = removeBullet ? (sign === 1 ? 0 : this.bulletsLength - 1) : -1;
          const bullets = [].slice.call(this.ref.querySelectorAll('span'));
          bullets.forEach((bullet, index) => {
              this.handleBulletMovement({ bullet, index, sign, removeBullet, removeBulletIndex, activeIndex: mappedActiveIndex });
          });
          // append or prepend new bullet
          if (removeBullet) {
              const insertBulletIndex = -1 + this.bulletsLength - removeBulletIndex;
              const realIndex = getRealIndex(insertBulletIndex, this.currentIndex, mappedActiveIndex);
              const isEdge = isEdgeBullet(insertBulletIndex, realIndex, this.bulletsLength, this.totalSlides);
              const bullet = this.renderBullet(insertBulletIndex, { isEdge });
              this.insertBullet(sign, bullet);
          }
      }
      handleBulletMovement({ bullet, index, activeIndex, sign, removeBullet, removeBulletIndex, }) {
          if (removeBulletIndex === index) {
              return remove(bullet);
          }
          // shift index due to remove bullet
          index = index - (removeBullet ? sign : 0);
          const realIndex = getRealIndex(index, this.currentIndex, activeIndex);
          const isEdge = isEdgeBullet(index, realIndex, this.bulletsLength, this.totalSlides);
          this.setAttributes(bullet, {
              isEdge,
              isActive: index === activeIndex,
              position: removeBullet ? index * this.diameter : undefined,
          });
      }
      renderBullet(index, { isActive, isEdge } = {}) {
          const element = createElement('span', { classNames: 'virchual__pagination-bullet' });
          this.setAttributes(element, { isActive, isEdge, position: index * this.diameter });
          return element;
      }
      setAttributes(bullet, { isActive, isEdge, position }) {
          addOrRemoveClass(bullet, 'virchual__pagination-bullet--active', !isActive);
          addOrRemoveClass(bullet, 'virchual__pagination-bullet--edge', !isEdge);
          if (position != null) {
              bullet.style.transform = `translateX(${position}px)`;
          }
      }
      insertBullet(sign, bullet) {
          const insert = sign === 1 ? append : prepend;
          insert(this.ref, bullet);
      }
  }

  /**
   * Virtual slide component.
   */
  class Slide {
      constructor(html, frame, settings) {
          this.frame = frame;
          this.settings = settings;
          this.isMounted = false;
          this.isActive = false;
          this.hasChanged = false;
          if (typeof html === 'string') {
              this.html = html;
              return;
          }
          this.ref = html;
          this.html = this.ref.innerHTML;
          this.isMounted = true;
      }
      set(property, value) {
          this[property] = value;
          this.hasChanged = true;
      }
      render() {
          this.ref = createElement('div', { classNames: 'virchual__slide', html: this.html });
          this.setAttributes();
          return this.ref;
      }
      mount(prepend$1 = false) {
          if (this.isMounted) {
              // slide has changed -> update in DOM
              if (this.hasChanged) {
                  this.update();
              }
              return;
          }
          console.debug('[Mount] Slide', { ref: this.ref, prepend: prepend$1 });
          this.render();
          this.isMounted = true;
          this.ref.addEventListener('transitionend', () => {
              this.ref.style.transition = '';
              this.transitionEndCallback();
          });
          const insert = prepend$1 ? prepend : append;
          insert(this.frame, this.ref);
      }
      unmount() {
          console.debug('[Unmount] Slide', { ref: this.ref });
          this.isMounted = false;
          remove(this.ref);
      }
      /**
       * Start transition.
       *
       * @param value Position to translate to.
       * @param easing Enable or disable transition easing.
       * @param done Callback function after transition has ended.
       */
      translate(value, { easing, done } = {}) {
          this.transitionEndCallback = done || noop;
          if (easing) {
              this.ref.style.transition = `transform ${this.settings['speed']}ms ${this.settings['easing']}`;
          }
          else {
              this.ref.style.transition = '';
          }
          this.ref.style.transform = `translate3d(calc(${this.position}% + ${Math.round(value)}px), 0, 0)`;
      }
      update() {
          this.hasChanged = false;
          this.setAttributes();
      }
      setAttributes() {
          addOrRemoveClass(this.ref, 'virchual__slide--active', !this.isActive);
          this.translate(0, { easing: true });
      }
  }

  class Virchual {
      constructor(container, settings = {}) {
          this.container = container;
          this.settings = settings;
          this.currentIndex = 0;
          this.slides = [];
          this.isBusy = false;
          this.frame = this.container.querySelector('.virchual__frame');
          assert(this.frame, 'Invalid element');
          this.currentIndex = 0;
          this.settings = Object.assign({ slides: [], speed: 200, swipeDistanceThreshold: 150, flickVelocityThreshold: 0.6, flickPower: 600, easing: 'ease-out', pagination: true, window: 1 }, settings);
          this.eventBus = new Event();
          this.eventBindings = {
              onClick: this.onClick.bind(this),
              onDrag: this.onDrag.bind(this),
              onDragEnd: this.onDragEnd.bind(this),
          };
          let rawSlides;
          const slides = this.settings['slides'];
          if (typeof slides === 'function') {
              rawSlides = slides();
          }
          else {
              rawSlides = slides;
          }
          this.slides = this.hydrate();
          this.slides = this.slides.concat((rawSlides || []).map(slide => new Slide(slide, this.frame, this.settings)));
          this.pagination = new Pagination(this.container, this.slides.length);
          this.pagination.render();
      }
      /**
       * Register component class.
       *
       * @param cls Component class.
       * @param settings Optional settings.
       */
      register(cls, settings) {
          new cls({ virchual: this, eventBus: this.eventBus }, settings);
      }
      /**
       * Mount components.
       */
      mount() {
          console.debug('[Mount] Virchual');
          this.mountAndUnmountSlides();
          this.bindEvents();
          new Drag(this.frame, { event: this.eventBus }).mount();
          this.eventBus.emit('mounted');
      }
      /**
       * Disable Virchual instance and all user interactions (changing slides)
       *
       * @TODO
       */
      disable() {
          return;
      }
      /**
       * Enable previously disabled Virchual instance. Enable user interaction.
       *
       * @TODO
       */
      enable() {
          return;
      }
      /**
       * Register callback fired on the given event(s).
       *
       * @param events An event name. Use space to separate multiple events.
       *                             Also, namespace is accepted by dot, such as 'resize.{namespace}'.
       * @param handler A callback function.
       * @param elm  Optional. Native event will be listened to when this arg is provided.
       * @param opts Optional. Options for addEventListener.
       */
      on(events, handler) {
          this.eventBus.on(events, handler);
      }
      /**
       * Unsubscribe the given event.
       *
       * @param events - A event name.
       * @param elm    - Optional. removeEventListener() will be called when this arg is provided.
       */
      off(events) {
          this.eventBus.off(events);
      }
      /**
       * Go to previous slide.
       */
      prev() {
          console.debug('[Controls] Previous');
          this.goTo('prev');
      }
      /**
       * Go to next slide.
       */
      next() {
          console.debug('[Controls] Next');
          this.goTo('next');
      }
      /**
       * Destroy instance, remove DOM events.
       */
      destroy() {
          this.eventBus.destroy();
          this.eventBus.emit('destroy');
      }
      goTo(control) {
          const slide = this.slides[this.currentIndex];
          slide.translate(-100, {
              easing: true,
              done: () => {
                  this.isBusy = false;
              },
          });
          const sign = control === 'prev' ? -1 : +1;
          this.currentIndex = rewind(this.currentIndex + sign * 1, this.slides.length - 1);
          this.mountAndUnmountSlides({ control: control });
          const move = control === 'prev' ? this.pagination.prev.bind(this.pagination) : this.pagination.next.bind(this.pagination);
          move();
          this.eventBus.emit('move', { index: this.currentIndex, control });
      }
      hydrate() {
          const slideElements = [].slice.call(this.frame.querySelectorAll('div'));
          return slideElements.map(element => new Slide(element, this.frame, this.settings));
      }
      /**
       * Mount and unmount slides.
       */
      mountAndUnmountSlides({ control } = {}) {
          const currentSlide = this.slides[this.currentIndex];
          const indices = range(0, this.slides.length - 1);
          const window = this.settings['window'];
          const mountableSlideIndices = slidingWindow(indices, this.currentIndex, window);
          const mountableSlideIndicesWithOffset = slidingWindow(indices, this.currentIndex, window + 1);
          mountableSlideIndicesWithOffset.forEach(index => {
              const slide = this.slides[index];
              if (index === this.currentIndex) {
                  currentSlide.set('isActive', true);
              }
              else {
                  currentSlide.set('isActive', false);
              }
              const realIndex = mountableSlideIndices.indexOf(index);
              // unmount
              if (realIndex < 0) {
                  return slide.unmount();
              }
              slide.set('position', (window - realIndex) * -100);
              const prepend = control === 'prev' || (control == null && this.slides[0].isMounted && realIndex - window < 0);
              slide.mount(prepend);
          });
      }
      bindEvents() {
          this.eventBus.on('drag', this.eventBindings.onDrag);
          this.eventBus.on('dragend', this.eventBindings.onDragEnd);
          this.eventBus.on('click', this.eventBindings.onClick, this.frame, { capture: true });
      }
      /**
       * Called when frame is clicked.
       *
       * @param event A click event.
       */
      onClick(event) {
          this.isBusy && stop(event);
      }
      /**
       * Handle drag event.
       *
       * @param event
       */
      onDrag(event) {
          this.isBusy = true;
          const mountableSlideIndices = slidingWindow(range(0, this.slides.length - 1), this.currentIndex, this.settings['window']);
          const sign = event.control === 'prev' ? +1 : -1;
          mountableSlideIndices.forEach(index => {
              const slide = this.slides[index];
              const x = sign * Math.abs(event.offset.x);
              slide.translate(x);
          });
      }
      /**
       * Handle dragged event.
       *
       * @param event
       */
      onDragEnd(event) {
          console.debug('[Drag] Drag end', event);
          this.goTo(event.control);
      }
  }

  class Virchual$1 extends Virchual {
      constructor(container, settings = {}) {
          super(container, settings);
          this.register(Controls, { isEnabled: true });
          // this.register(Lazy, { threshold: 300 }); @TODO
          this.register(LazyLoadImage, { lazyload: true });
      }
  }

  return Virchual$1;

})));
//# sourceMappingURL=virchual.bundle.js.map
